From fe160d67d36c9b705b110872e597f1febfac636e Mon Sep 17 00:00:00 2001
From: Alyssa Rosenzweig <alyssa@rosenzweig.io>
Date: Thu, 5 Oct 2023 17:44:09 -0400
Subject: [PATCH] nir/opt_algebraic: Optimize LLVM booleans

Like D3D booleans but needlessly different. Helps OpenCL kernels.

Signed-off-by: Alyssa Rosenzweig <alyssa@rosenzweig.io>
---
 src/compiler/nir/nir_opt_algebraic.py | 4 ++++
 1 file changed, 4 insertions(+)

diff --git a/src/compiler/nir/nir_opt_algebraic.py b/src/compiler/nir/nir_opt_algebraic.py
index 54e119cbbee..2881c6c7054 100644
--- a/src/compiler/nir/nir_opt_algebraic.py
+++ b/src/compiler/nir/nir_opt_algebraic.py
@@ -1508,6 +1508,10 @@ optimizations.extend([
    (('umax', ('b2i', 'a@1'), ('b2i', 'b@1')), ('b2i', ('ior',  a, b))),
    (('umin', ('b2i', 'a@1'), ('b2i', 'b@1')), ('b2i', ('iand', a, b))),
 
+   # LLVM booleans look like D3D booleans but with iand instead of ineg. Lower
+   # LLVM booleans to D3D booleans so the above rules kick in.
+   (('iand', ('b2i', a), 1), ('ineg', ('b2i', a))),
+
    (('ine', ('umin', ('ineg', ('b2i', 'a@1')), b), 0), ('iand', a, ('ine', b, 0))),
    (('ine', ('umax', ('ineg', ('b2i', 'a@1')), b), 0), ('ior' , a, ('ine', b, 0))),
 
-- 
2.41.0

